#!/usr/bin/env python
# Get LJ comment stats, and print them with some familiar format.
# (C) 2004 kvance.  Distribute under GNU GPL.
# <mailto:kvance@kvance.com>
#
# 1.0 - initial release

import os, re, sys, urllib, urllib2, xml.sax
from math   import log
from sys    import stderr
from time   import ctime
from xml.sax import ContentHandler, saxutils
from threading import Timer

###
###  CONSTANTS
###

VERSION = "1.0"

# "Animation" sequence :D
awesome_animation = ('/','-','\\','|')

# LJ export comments URI
COMMENTS_URI = \
    'http://www.livejournal.com/export_comments.bml?get=comment_meta&startid=0'

# LJ login URI
LOGIN_URI = 'https://www.livejournal.com/login.bml'

###
###  BUSY ANIMATION
###
class Busy:
    def __init__(self, message):
        self.anim_pos = 0
        self.delay    = 0.3
        stderr.write(message+': |')
        stderr.flush()
        self.timer = Timer(self.delay, self.anim_step)
        self.timer.start()

    def stop(self):
        self.timer.cancel()
        stderr.write("\b \b\b\b.\n")
        stderr.flush()

    def anim_step(self):
        stderr.write("\b"+awesome_animation[self.anim_pos])
        stderr.flush()
        self.anim_pos = (self.anim_pos+1) % len(awesome_animation)
        # Apparently, you can't just have a repeating timer
        self.timer = Timer(self.delay, self.anim_step)
        self.timer.start()

###
###  COMMENT SORTER / CHEAP SAX PARSER
###
class CommentHandler(saxutils.DefaultHandler):
    def __init__(self):
        self.poster_count = {}   # Map ID to comment count
        self.poster_name  = {'Anonymous':'Anonymous'}   # Map ID to name

    # SAX la-z parser
    def startElement(self, name, attrs):
        if name == 'comment':
            # Get poster ID
            id = attrs.get('posterid')
            if id == None: id = 'Anonymous'
            else:          id = int(id)
            # Increment her post count
            if not self.poster_count.has_key(id): self.poster_count[id]  = 1
            else:                                 self.poster_count[id] += 1
                
        elif name == 'usermap':
            id = int(attrs.get('id'))
            self.poster_name[id] = attrs.get('user')

    # Return the top (user,post count) pairs
    def top_posters(self):
        # Sort ids by comment count, descending.
        poster_ids = self.poster_count.keys()
        poster_ids.sort(
                lambda x,y: cmp(self.poster_count[x], self.poster_count[y]))
        poster_ids.reverse()

        # Iterate over the sorted list
        for id in poster_ids:
            yield (self.poster_name[id], self.poster_count[id])

    def total_comments(self):
        return sum(self.poster_count.values())

    def total_commenters(self):
        return len(self.poster_name.keys())+1

    # Highest number of comments
    def highest_posts(self):
        name,count = self.top_posters().next()
        return count
# end CommentHandler

###
###  HTTP FUNCTIONS
###
# Login to LJ and get a cookie.
def get_lj_cookies(username, passwd):
    # Encode HTTP request
    args = urllib.urlencode( {'user': username, 'password': passwd} )
    # Send HTTP request
    try:
        cookie_enc = urllib.urlopen(LOGIN_URI, args).info()['Set-Cookie']
    except:
        return None
    # Fish out an ljsession cookie
    cookies    = [c for c in cookie_enc.split() if c.startswith('ljsession')]
    if len(cookies) == 0:
        return None
    else:
        return cookies[0][:-1]

# Trade in that cookie for some metadata.
def get_lj_comment_data(cookie):
        # Encode HTTP request
        headers = {'Cookie': cookie}
        request = urllib2.Request(url=COMMENTS_URI, headers=headers)
        # Send HTTP request
        connection = urllib2.urlopen(request)
        # Return the response
        return connection.read()

###
###  COMMANDLINE FUNCTIONS
###
def print_usage():
    print >>stderr, 'USAGE: lj_comment_stats <username> <password> [options]'
    print >>stderr, ''
    print >>stderr, """Options:
    -fancytext      Output in fancy ASCII text  (default)
    -plaintext      Output in boring ASCII text
    -html           Output in familiar LiveJournalified HTML
    -topN           Show the top N commenters   (default -top10)
    -maxN           Limit to N comments, placing an lj-cut after the top N."""

# Return a list of arguments and options from the commandline
def parse_cmdline(argv):
    # Defaults
    prefs = { 'style': 'fancytext',
              'top_n': 10,
            }

    # Split options and args
    options = filter(lambda o:    o.startswith('-'), argv)
    args    = filter(lambda a:not a.startswith('-'), argv)

    # Check opts
    for o in options:
        # Style option
        if o=='-fancytext' or o=='-plaintext' or o=='-html':
            prefs['style'] = o[1:]
        # Number option
        elif o.startswith('-top') or o.startswith('-max'):
            try:
                n = int(o[4:])
                prefs[o[1:4]+'_n'] = n
            except ValueError:
                print_usage()
                print >>stderr, "\nI'm sorry.  I don't understand your number" \
                                " for "+o[0:4]+"N ("+o[4:]+")."
                sys.exit(1)
        else:
            print_usage()
            print >>stderr, "\nI'm sorry.  I don't understand your option '" \
                            +o+"'"
            sys.exit(1)

    # Check args
    if len(args) != 2:
        print_usage()
        print >>stderr, "\nI'm sorry.  I only understood "+`len(args)`+\
                        " arguments, and I require 2."
        sys.exit(0)

    return (prefs, args)
# end parse_cmdline

###
###  OUTPUT FUNCTIONS
###
def output_header(style, username):
    heading = 'LJ Comment stats for '
    dashes  = '---------------------'
    for i in range(0, len(username)):
        dashes += '-'

    if style == 'plaintext':
        print heading+username
    if style == 'fancytext':
        print str(heading+username).center(50)
        print  dashes.center(50)+"\n"
    if style == 'html':
        print '<font size="+1">'+heading+'<lj user='+username+'></font><br>'
        print '<table>'

def output_row(style, n, user, count, total):
    # Logarithmic scale seems to look nice
    percent = int(log(1+(float(count)/total*49), 50)*100)
    width   = percent*4

    if style == 'plaintext':
        print `n`+'. '+user+': '+`count`
    if style == 'fancytext':
        stars = ''
        for i in range(0, int(percent/5)):
            stars += '*'
        print str(n).center(4)+' '+user.rjust(16)+' ' \
              +stars.ljust(20)+' '+`count`
    if style == 'html':
        print '<tr><td><b>'+`n`+'</b></td><td><lj user="'+user+'"></td>'
        print """<td><img src="http://stat.livejournal.com/img/poll/leftbar.gif"
align="absmiddle" height="14" width="7"><img src=
"http://stat.livejournal.com/img/poll/mainbar.gif" align=
"absmiddle" height="14" width="""+`width`+' alt="'+`count`+'"><img src='+\
""""http://stat.livejournal.com/img/poll/rightbar.gif" align=
"absmiddle" height="14" width="7"> <b>"""+`count`+'</b></td></tr>'

def output_cut(style):
    if style == 'plaintext':
        print '--- CUT ---'
    if style == 'fancytext':
        print '---8<--------8<--------  cut here  ---8<--------8<--------8<---'
    if style == 'html':
        print '<tr><td colspan="3"><lj-cut></td></tr>'

def output_footer(style, total_comments, total_commenters, max_n):
    if style == 'plaintext' or style == 'fancytext':
        print "\nTotal Commenters: "+`total_commenters`
        print "Total Comments: "+`total_comments`
        print "\nReport generated "+ctime()+" by kvance's lj_comment_stats " \
              +VERSION
    if style == 'html':
        if max_n != None:
            print '</lj-cut>'
        print '</table><br>'
        print '<font size="+1">Total Commenters: '+`total_commenters`+'<br>'
        print 'Total Comments: '+`total_comments`+'</font><br>'
        print '<small>Report generated '+ctime()+' by <lj user="kvance">\'s '\
              +'lj_comment_stats '+VERSION+'</small>'

###
###  ENTRY POINT
###
if __name__ == "__main__":
    print >>stderr, \
            "lj_comment_stats "+VERSION+" (C) 2004 kvance.  Distribute under "\
            "GNU GPL."

    # Parse the commandline
    prefs, args = parse_cmdline(sys.argv[1:])

    # Log in
    username,passwd = args
    b = Busy('Logging in')
    cookie = get_lj_cookies(username, passwd)
    b.stop()
    if cookie == None or cookie == 'ljsession=':
        print >>stderr, 'Login has failed.  Please forgive me; I am quitting.'
        sys.exit(1)

    # Fetch comment metadata
    b = Busy('Downloading comment data')
    data = get_lj_comment_data(cookie)
    b.stop()

    # Parse the comment data
    comms = CommentHandler()
    xml.sax.parseString(data, comms)

    # Get stats
    total_comments   = comms.total_comments()
    total_commenters = comms.total_commenters()
    highest          = comms.highest_posts()
    style            = prefs['style']
    top_n            = prefs['top_n']
    if prefs.has_key('max_n'):  max_n = prefs['max_n']
    else:                       max_n = None

    # Output result
    n = 1
    output_header(style, username)
    for user,posts in comms.top_posters():
        output_row(style, n, user, posts, highest)
        if max_n != None:
            if n == top_n: output_cut(style)
            if n == max_n: break
        elif   n == top_n: break
        n += 1
    output_footer(style, total_comments, total_commenters, max_n)
